#ifndef SHARED_MEMORY_H
#define SHARED_MEMORY_H

#include <string>
#include <rtm/ConnectorListener.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <boost/interprocess/managed_windows_shared_memory.hpp>


namespace cv {
	class Mat;
};

namespace RTC {
	struct Timed_cvMat;
	class OutPortConnector;
};

namespace USM{
	// |[gڑɌĂ΂R[obNp̃NX
	class OnConnectedListener : public RTC::ConnectorListener
	{
	private:
		bool &useSharedMemory;
		std::string my_ip;

	public:
		OnConnectedListener(bool &useSharedMemory, std::string my_ip);
		void operator()(const RTC::ConnectorInfo& info);
	};


	// |[gؒfɌĂ΂R[obNp̃NX
	class OnDisconnectedListener : public RTC::ConnectorListener
	{
	private:
		bool &useSharedMemory;
		std::string &my_ip;
		std::vector<RTC::OutPortConnector*> &m_connectors;

	public:
		OnDisconnectedListener(bool &useSharedMemory, std::string &my_ip, std::vector<RTC::OutPortConnector*> &m_connectors);
		void operator()(const RTC::ConnectorInfo& info);
	};


	// RTC::OutPortpo̓|[g
	template <class DataType>
	class OutPort
		: public RTC::OutPort<DataType>
	{
		DataType& m_value;

	private:
		// IORIPAhX𒊏o
		std::string getIpFromIOR(omniIOR &ior);
		
		// gIPAhX
		std::string my_ip;

		boost::interprocess::managed_windows_shared_memory m_shared_memory;

		// Lgۂ̃tO
		bool useSharedMemory;

		// L̃Oobt@
		unsigned int m_idx_num;

		// ݐ̋L̃CfbNXԍ
		unsigned int m_curr_idx_num;

		// obt@̃TCY
		unsigned int m_buffer_size;

		// Oobt@ێ邽߂̃}bv
		std::map<std::string, unsigned char*> m_memory_map;

		// L
		std::string m_memory_name;

		// cv::MatTimed_cvMatւ̕ϊiLւ̏݊܂ށj
		void cvMat2Timed_cvMat(cv::Mat src, RTC::Timed_cvMat &dst);


	public:
		OutPort(const char* name, DataType& value);
		~OutPort(){};

		// L쐬p̏֐
		void initialize();

		// RTC::OutPort.write()Ɠl̋鏑݊֐
		bool write();
		bool write(RTC::Timed_cvMat src);

		// cv::MatTimed_cvMatւ̕ϊsCOutPortɏފ֐
		bool write(cv::Mat src);

		// IɋL̎gpۂύX֐ifobOpj
		void setMemoryFlag(bool flag);
	};


	// RTC::InPortp̓|[g
	template <class DataType>
	class InPort
		: public RTC::InPort<DataType>
	{
		DataType& m_value;

	public:
		InPort(const char* name, DataType& value);
		~InPort(){};

		// RTC::InPort.read()Ɠl̋ǂݍ݊֐
		bool read();

		// InPortǂݍ݁CTimed_cvMatcv::Matւ̕ϊs֐
		bool read(cv::Mat& dst);
	};


	// Oobt@̃ANZX߂֐
	std::string createIndex(int idx);
};

#endif  // SHARED_MEMORY_H